/* Written by Paul Eggert.  */

#include "filetype.h"

char const *
file_type(struct stat const *st) {
    /* To keep diagnostics grammatical in English, the returned string
       must start with a consonant.  */

    /* Do these three first, as they're the most common.  */

    if (S_ISREG (st->st_mode))
        return st->st_size == 0 ? ("regular empty file") : ("regular file");

    if (S_ISDIR (st->st_mode))
        return ("directory");

    if (S_ISLNK (st->st_mode))
        return ("symbolic link");

#if 0
    /* Do the |S_TYPEIS*| macros next, as they may be implemented in
       terms of |S_ISNAM|, and we want the more-specialized
       interpretation.  */

    if (S_TYPEISMQ (st))
        return ("message queue");

    if (S_TYPEISSEM (st))
        return ("semaphore");

    if (S_TYPEISSHM (st))
        return ("shared memory object");

    if (S_TYPEISTMO (st))
        return ("typed memory object");

    /* The remaining are in alphabetical order.  */

    if (S_ISBLK (st->st_mode))
        return ("block special file");

    if (S_ISCHR (st->st_mode))
        return ("character special file");

    if (S_ISCTG (st->st_mode))
        return ("contiguous data");

    if (S_ISFIFO (st->st_mode))
        return ("fifo");

    if (S_ISDOOR (st->st_mode))
        return ("door");

    if (S_ISMPB (st->st_mode))
        return ("multiplexed block special file");

    if (S_ISMPC (st->st_mode))
        return ("multiplexed character special file");

    if (S_ISMPX (st->st_mode))
        return ("multiplexed file");

    if (S_ISNAM (st->st_mode))
        return ("named file");

    if (S_ISNWK (st->st_mode))
        return ("network special file");

    if (S_ISOFD (st->st_mode))
        return ("migrated file with data");

    if (S_ISOFL (st->st_mode))
        return ("migrated file without data");

    if (S_ISPORT (st->st_mode))
        return ("port");

    if (S_ISSOCK (st->st_mode))
        return ("socket");

    if (S_ISWHT (st->st_mode))
        return ("whiteout");
#endif

    return ("weird file");
}
